let resources_url = "https://flexa-releases.s3.eu-central-1.amazonaws.com";
// let resources_url = "http://localhost:8000";
let os_name = null;

async function download_batch(from_i, to_i, remote_manifest_content, download_info, local_manifest_content) {
    for (let i = from_i; i < to_i; i++) {

        if (remote_manifest_content.files[i] == undefined) {
            break;
        }

        let file = remote_manifest_content.files[i];
        console.log("Downloading " + file.path);
        let requires_update = null;
        for (let local_manifest_file of local_manifest_content.files) {
            if (local_manifest_file.path == file.path) {
                if (local_manifest_file.hash != file.hash) {
                    requires_update = true;
                } else {
                    requires_update = false;
                }
                break;
            }
        }

        if (requires_update == false) {
            continue;
        }

        document.querySelector("#file").innerHTML = file.path.split("/").reverse()[0];

        document.querySelector("#counter").innerHTML = Math.round(download_info.downloaded_count / remote_manifest_content.files.length * 100) + "%";

        let file_response = await fetch(resources_url + file.path);
        let reader = file_response.body.getReader();
        let total_size = parseInt(file_response.headers.get("Content-Length"));
        let downloaded_size = 0;
        let downloaded_data = [];

        while (1) {
            let status = await reader.read();

            if (status.done == true) {
                downloaded_size = total_size;
                break;
            }

            downloaded_size += status.value.length;
            downloaded_data.push(status.value);

            document.querySelector("#counter").innerHTML = Math.round(downloaded_size / total_size * 100) + "%";
        }

        let file_content = new Blob(downloaded_data);

        await window.node.send_message({
            action: "put_file",
            path: file.path,
            content: await file_content.arrayBuffer()
        });

        download_info.downloaded_count++;

    }
}

async function on_load() {

    /*window.node.on_message("alert", (sender, message) => {
        alert(message);
    });*/


    let os_name = await window.node.send_message({
        action: "get_os_name"
    });

    resources_url += "/" + os_name;

    let app_path = await window.node.send_message({
        action: "get_app_path"
    });
    document.querySelector("#path").innerHTML = app_path;

    await window.node.send_message({
        action: "create_app_folder"
    });



    let manifest_response = await fetch(resources_url + "/manifest.json?t=" + Date.now());
    let remote_manifest_content = await manifest_response.json();

    let force_update = false;
    let local_manifest_content = await window.node.send_message({ action: "read_file", path: "/manifest.json" });
    if (local_manifest_content === undefined) {
        local_manifest_content = { creation_time: 0, files: [] };
        force_update = true;
    } else {
        local_manifest_content = JSON.parse(local_manifest_content);
    }

    if (remote_manifest_content.creation_time != local_manifest_content.creation_time) {
        let want_update = true;

        if (force_update == false) {
            want_update = confirm("A new version is available. Would you like to download the update?");
        }

        if (want_update) {

            let batch_count = 20;
            let batch_size = Math.ceil(remote_manifest_content.files.length / batch_count);
            let promises = [];
            let download_info = { downloaded_count: 0 };
            for (let i = 0; i < batch_size; i++) {
                let p = download_batch(i * batch_size, (i + 1) * batch_size, remote_manifest_content, download_info, local_manifest_content);
                promises.push(p);
            }

            await Promise.all(promises);
            await window.node.send_message({
                action: "put_file",
                path: "/manifest.json",
                content: JSON.stringify(remote_manifest_content)
            });

            await window.node.send_message({ action: "unzip_file" });
        }
    }


    await window.node.send_message({ action: "start_app" });
}

window.addEventListener("load", on_load);
