const { app, BrowserWindow, ipcMain } = require('electron')
const path = require('path')
const fs = require("fs");
let os = require("os");
let app_path = app.getAppPath();

// making the npm start and the exe running the same
app_path = app_path.replaceAll(path.sep, "/");
app_path = app_path.replaceAll("/resources/app", "");

// OSX dist
app_path = app_path.replaceAll("/Flexa Launcher.app/Contents/Resources/app", "");

app_path = app_path.replaceAll("/", path.sep);

const flexa_app_path = app_path + path.sep + "flexa_app";
let main_window = null;

ipcMain.handle("ping", (event) => {
    return "pong";
});

ipcMain.handle("main", async (event, message) => {
    if (message.action == "create_app_folder") {
        try {
            fs.mkdirSync(flexa_app_path);
        } catch (e) {}
    }

    if (message.action == "get_app_path") {
        return app_path;
    }

    if (message.action == "get_os_name") {
        return os.platform().indexOf("win32") > -1 ? "win" : "osx";
    }

    if (message.action == "read_file"){
        let os_path = message.path.replace("/", path.sep);
        let file_contents = undefined;
        if(fs.existsSync(flexa_app_path+os_path)){
            file_contents = fs.readFileSync(flexa_app_path + os_path, {encoding: "utf8"});
        }
        return file_contents;
    }

    if (message.action == "put_file") {
        let os_path = message.path.replace("/", path.sep);
        // console.log("saving " + flexa_app_path + os_path);
        let folder = path.dirname(os_path);
        if (!fs.existsSync(flexa_app_path + folder)) {
            fs.mkdirSync(flexa_app_path + folder, { recursive: true });
        }
        try {
            fs.writeFileSync(flexa_app_path + os_path, Buffer.from(message.content));
        } catch (e) {
            console.log("Error during save " + os_path + ", skipping");
        }
    }

    if (message.action == "unzip_file") {
        const zl = require("zip-lib");
        await zl.extract(flexa_app_path + path.sep + "flexa.zip", flexa_app_path);
    }

    if (message.action == "start_app") {
        main_window.hide();
        var child_process = require('child_process');
        var executablePath = flexa_app_path + path.sep + "Flexa.exe";
        if (!fs.existsSync(executablePath)) {
            executablePath = flexa_app_path + path.sep + "Flexa.app/Contents/MacOS/Flexa";
        }
        let child = child_process.execFile(executablePath, function(error, stdOut, stdErr) { console.log({error, stdOut, stdErr}); });
        child.on("close", (event) => {
            app.quit();
        });
    }
});

function createWindow() {
    const win = new BrowserWindow({
        width: 500,
        height: 300,
        // resizable: false,
        webPreferences: {
            preload: path.join(__dirname, 'preload.js')
        },
        icon: path.join(__dirname, 'favicon.ico')
    });

    if (main_window === null) {
        main_window = win;
    }

    win.loadFile('src/index.html');
    win.menuBarVisible = false;
    // win.webContents.openDevTools();
}

app.whenReady().then(() => {
    createWindow()

    /*app.on('activate', () => {
        if (BrowserWindow.getAllWindows().length === 0) {
            createWindow()
        }
    })*/
})

app.on('window-all-closed', () => {
    app.quit();
})
